"""Exporters for the Data Docs report pipeline.

Exporters convert rendered HTML to various output formats.

Available Exporters:
- HtmlExporter: HTML output (default)
- PdfExporter: PDF output with optimized rendering
- MarkdownExporter: Markdown output
- JsonExporter: JSON output (structured data)
"""

from truthound.datadocs.exporters.base import (
    Exporter,
    BaseExporter,
    ExportResult,
    ExportOptions,
)
from truthound.datadocs.exporters.html import (
    HtmlExporter,
)
from truthound.datadocs.exporters.pdf import (
    PdfExporter,
    OptimizedPdfExporter,
    PdfOptions,
)
from truthound.datadocs.exporters.markdown import (
    MarkdownExporter,
)
from truthound.datadocs.exporters.json_exporter import (
    JsonExporter,
)

__all__ = [
    # Base
    "Exporter",
    "BaseExporter",
    "ExportResult",
    "ExportOptions",
    # HTML
    "HtmlExporter",
    # PDF
    "PdfExporter",
    "OptimizedPdfExporter",
    "PdfOptions",
    # Markdown
    "MarkdownExporter",
    # JSON
    "JsonExporter",
]
